--[[

ElvUI Location Plus
an addon for ElvUI, that adds player location and coords + 2 Datatexts
by Benik @ Emerald Dream EU
----------------------------------------------------------------------------------
   Hints: 
a. Click to Toggle WorldMap
b. ShiftClick to send your location in chat
c. CtrlClick to toggle left/right datatext panels
----------------------------------------------------------------------------------
   Credits:
- Elv, Blazeflack, Darth Predator, Talryn, for showing me the best way to do this
- Phenox, Blacknet, CohesioN`(idea from his TukUI edit)
- CodeRedLin, iceeagle, daf2(from Broker_Location addon)
for digging their great code and making this possible.
- Tuk, for making BlizzUI playable and Elv's forums community.
----------------------------------------------------------------------------------
	ToDo:
-- Less reload ui when changing layout
]]--

local E, L, DF = unpack(ElvUI); --Inport: Engine, Locales, ProfileDB, GlobalDB
local DT = E:GetModule('DataTexts')
local M = E:GetModule('Minimap')
local LSM = LibStub("LibSharedMedia-3.0")

local tourist = LibStub("LibTourist-3.0")

local LOCPANEL_HEIGHT = 22 --all panels height
local LOCPANEL_WIDTH = 200 --Location panel width, though its size depends on the location string
local COORDS_WIDTH = 30    --Coord panels width
local DTPANELS_WIDTH = 100 --Datatext panels width

local ElvUI_Location = CreateFrame('Frame', 'ElvUI_Location')
ElvUI_Location.version = '1.09'

-- mouse over the location panel
local function LocPanel_OnEnter(self,...)
	if loc_db.loc_shadow then
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOM", 0, -3); -- new tooltip position when shadows enabled
	else
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOM", 0, -1); -- default tooltip position
	end
	GameTooltip:ClearAllPoints()
	GameTooltip:SetPoint("BOTTOM", self, "BOTTOM", 0, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddDoubleLine(L["Zone : "], GetRealZoneText(),1,1,1,selectioncolor)
	GameTooltip:AddDoubleLine(L["Continent : "], tourist:GetContinent(GetRealZoneText()),1,1,1,selectioncolor)
	
	if loc_db.loc_tt then
		if loc_db.loc_tthint then
			GameTooltip:AddLine(" ")
			GameTooltip:AddDoubleLine(L["Click : "], L["Toggle WorldMap"],0.7,0.7,1,0.7,0.7,1)
			GameTooltip:AddDoubleLine(L["ShiftClick : "], L["Send position to chat"],0.7,0.7,1,0.7,0.7,1)
			GameTooltip:AddDoubleLine(L["CtrlClick : "], L["Toggle Datatexts"],0.7,0.7,1,0.7,0.7,1)
		end
		GameTooltip:Show()
		else
		GameTooltip:Hide()
	end
end

-- mouse leaving the location panel
local function LocPanel_OnLeave(self,...)
	GameTooltip:Hide()
end

-- clicking the location panel
local function LocPanel_OnClick(self,...)
	if IsShiftKeyDown() then
		local edit_box = _G.ChatEdit_ChooseBoxForSend()
		local x, y = GetPlayerMapPosition("player")
		local message
		local coords = string.format("%.0f, %.0f", x * 100, y * 100)
			if GetRealZoneText() ~= GetSubZoneText() then
				message = string.format("%s: %s (%s)", GetRealZoneText(), GetSubZoneText(), coords)
			else
				message = string.format("%s (%s)", GetRealZoneText(), coords)
			end
		_G.ChatEdit_ActivateChat(edit_box)
		edit_box:Insert(message) 
	else
		if IsControlKeyDown() then
			if loc_db.loc_datatxt then
				ToggleFrame(LeftCoordDtPanel)
				ToggleFrame(RightCoordDtPanel)
			else
				StaticPopup_Show("LOC_DT_DISABLED")
			end
		else 
			ToggleFrame(WorldMapFrame)
		end
	end
end

-- Location panel
function CreateMainLocPanel()
	local loc_panel = CreateFrame('Button', 'LocationPlusPanel', E.UIParent)
	--loc_panel:CreateBackdrop('Transparent')
	--loc_panel.backdrop:SetAllPoints()
	--if loc_db.loc_trans == false then
		--loc_panel:SetTemplate('Default', true)	
	--else
		--loc_panel:SetTemplate('Transparent')
	--end

	loc_panel:Width(LOCPANEL_WIDTH)
	loc_panel:Height(LOCPANEL_HEIGHT)

	loc_panel:Point('TOP', E.UIParent, 'TOP', 0, -E.mult-20)
	loc_panel:SetFrameStrata('LOW')
	loc_panel:RegisterForClicks('AnyUp')
	loc_panel:SetScript('OnEnter', LocPanel_OnEnter) 
	loc_panel:SetScript('OnLeave', LocPanel_OnLeave)
	loc_panel:SetScript('OnClick', LocPanel_OnClick)

	-- Location Text
	LocationPlusPanel.Text = LocationPlusPanel:CreateFontString(nil, "LOW")
	LocationPlusPanel.Text:Point("CENTER", 0, 0)
	LocationPlusPanel.Text:SetJustifyH('CENTER')
	LocationPlusPanel.Text:SetJustifyV('MIDDLE')
	
	loc_panel:SetScript("OnUpdate", function(self,event,...)
		local subZoneText = GetMinimapZoneText() or ""
		local zoneText = GetRealZoneText()
		local displayLine	
		-- zone and subzone
		if loc_db.loc_both then
			if (subZoneText ~= "") and (subZoneText ~= zoneText) then
				displayLine = zoneText .. ": " .. subZoneText
			else
				displayLine = subZoneText
			end
		else
			displayLine = subZoneText
		end

		LocationPlusPanel.Text:SetTextColor(M:GetLocTextColor())
		LocationPlusPanel.Text:SetText(displayLine)
		LocationPlusPanel:SetWidth(LocationPlusPanel.Text:GetWidth() + 24)
	end)
	
	--Hide in combat
	loc_panel:SetScript("OnEvent",function(self, event)
		if loc_db.loc_combat then
			if event == "PLAYER_REGEN_DISABLED" then
				UIFrameFadeOut(LocationPlusPanel, 0.2, LocationPlusPanel:GetAlpha(), 0)
				LocationPlusPanel.fadeInfo.finishedFunc = OnFade
			else
				loc_panel:Show()	
			end
			if event == "PLAYER_REGEN_ENABLED" then
				UIFrameFadeIn(LocationPlusPanel, 0.2, LocationPlusPanel:GetAlpha(), 1)
				LocationPlusPanel:Show()
			end
		end
	end)

	-- Mover
	E:CreateMover(LocationPlusPanel, "LocationMover", "Location Panel")
end

-- Update changes
function LocPlusUpdate()
	LocPanelHeight()
	AllLocPanelsTransparent()
	AllLocPanelsShadow()
	AllLocPanelsShadowReposition()
end

-- Fonts -Credits: Phenox(with support for his addon)
function E:LocCoordPanelFont()

	self["media"].lpFont = LSM:Fetch("font", loc_db.lpfont)
	LocationPlusPanel.Text:FontTemplate(E["media"].lpFont, loc_db.lpfontsize, loc_db.lpfontflags)			
	XCoordsPanel.Text:FontTemplate(E["media"].lpFont, loc_db.lpfontsize, loc_db.lpfontflags)
	YCoordsPanel.Text:FontTemplate(E["media"].lpFont, loc_db.lpfontsize, loc_db.lpfontflags)

	if not (IsAddOnLoaded("ElvUI_Phenox")) then
	local panelsToFont = { RightCoordDtPanel, LeftCoordDtPanel }
	for panelName, panel in pairs(panelsToFont) do
		for i=1, panel.numPoints do
			local pointIndex = DT.PointLocation[i]
			panel.dataPanels[pointIndex].text:FontTemplate(E["media"].lpFont, loc_db.lpfontsize, loc_db.lpfontflags)			
		end
	end	
	
	else return end

end

-- Change LocPanel Height
function LocPanelHeight()
	if loc_db.loc_ht then
		LocationPlusPanel:Height((LOCPANEL_HEIGHT)+6)  -- height option on Location Panel	
	else
		LocationPlusPanel:Height(LOCPANEL_HEIGHT) -- default height
	end
end

-- Enable/Disable shadows without reloadui, YEY!
function AllLocPanelsShadow()
	local panelsToAddShadow = { LocationPlusPanel, XCoordsPanel, YCoordsPanel, LeftCoordDtPanel, RightCoordDtPanel }
	
	for _, frame in pairs( panelsToAddShadow ) do
	frame:CreateShadow('Default')
		if loc_db.loc_shadow then
			frame.shadow:Show()
		else
			frame.shadow:Hide()
		end
	end
end

-- Reposition when shadows enabled
function AllLocPanelsShadowReposition()
	if loc_db.loc_shadow then
		XCoordsPanel:Point('RIGHT', LocationPlusPanel, 'LEFT', -3, 0)
		XCoordsPanel:Point('CENTER', LocationPlusPanel, 'CENTER', -3, 0)
		YCoordsPanel:Point('LEFT', LocationPlusPanel, 'RIGHT', 3, 0)
		YCoordsPanel:Point('CENTER', LocationPlusPanel, 'CENTER', 3, 0)
		LeftCoordDtPanel:Point('RIGHT', XCoordsPanel, 'LEFT', -3, 0)
		LeftCoordDtPanel:Point('CENTER', XCoordsPanel, 'CENTER', -3, 0)
		RightCoordDtPanel:Point('LEFT', YCoordsPanel, 'RIGHT', 3, 0)
		RightCoordDtPanel:Point('CENTER', YCoordsPanel, 'CENTER', 3, 0)
	else
		XCoordsPanel:Point('RIGHT', LocationPlusPanel, 'LEFT', -1, 0)
		XCoordsPanel:Point('CENTER', LocationPlusPanel, 'CENTER', -1, 0)
		YCoordsPanel:Point('LEFT', LocationPlusPanel, 'RIGHT', 1, 0)
		YCoordsPanel:Point('CENTER', LocationPlusPanel, 'CENTER', 1, 0)
		LeftCoordDtPanel:Point('RIGHT', XCoordsPanel, 'LEFT', -1, 0)
		LeftCoordDtPanel:Point('CENTER', XCoordsPanel, 'CENTER', -1, 0)
		RightCoordDtPanel:Point('LEFT', YCoordsPanel, 'RIGHT', 1, 0)
		RightCoordDtPanel:Point('CENTER', YCoordsPanel, 'CENTER', 1, 0)
	end
end

-- Attempt to toggle transparency, works 90%
function AllLocPanelsTransparent()
	local panelsToAddTrans = { LocationPlusPanel, XCoordsPanel, YCoordsPanel, LeftCoordDtPanel, RightCoordDtPanel }
	
	for _, frame in pairs( panelsToAddTrans ) do
		if loc_db.loc_trans then
			frame:SetTemplate('Transparent')
		else
			frame:SetTemplate('Default', true)
		end
	end
end

-- Hiding when entering combat
local function OnFade()
	LocationPlusPanel:Hide()
	LeftCoordDtPanel:Hide()
	RightCoordDtPanel:Hide()
end

-- Coord panels
function CreateLocCoordPanels()

	-- X Coord panel
	local coordsX = CreateFrame('Frame', "XCoordsPanel", LocationPlusPanel)
	--coordsX:SetTemplate('Default', true)
	coordsX:Width(COORDS_WIDTH)
	coordsX:Height(LOCPANEL_HEIGHT)
	coordsX:SetFrameStrata(LocationPlusPanel:GetFrameStrata())
	coordsX.Text = XCoordsPanel:CreateFontString(nil, "LOW")
	coordsX.Text:SetPoint("CENTER", 0, 0)
	coordsX.Text:SetJustifyH('CENTER')
	coordsX.Text:SetJustifyV('MIDDLE')	

	-- Y Coord panel
	local coordsY = CreateFrame('Frame', "YCoordsPanel", LocationPlusPanel)
	--coordsY:SetTemplate('Default', true)
	coordsY:Width(COORDS_WIDTH)
	coordsY:Height(LOCPANEL_HEIGHT)
	coordsY:SetFrameStrata(LocationPlusPanel:GetFrameStrata())
	coordsY.Text = YCoordsPanel:CreateFontString(nil, "LOW")
	coordsY.Text:SetPoint("CENTER", 0, 0)
	coordsY.Text:SetJustifyH('CENTER')
	coordsY.Text:SetJustifyV('MIDDLE')

	local ela,go = 0,false

	local cUpdate = function(self,t)
		ela = ela - t
		if ela > 0 then return end
			local x,y = GetPlayerMapPosition("player")
			local xt,yt
			x = math.floor(100 * x)
			y = math.floor(100 * y)
				if x == 0 and y == 0 then
					coordsX.Text:SetText("-")
					coordsY.Text:SetText("-")
				else
				if x < 10 then
					xt = "0"..x
				else
					xt = x
				end
			if y < 10 then
				yt = "0"..y
			else
				yt = y
			end
			coordsX.Text:SetText(xt)
			coordsY.Text:SetText(yt)
		end
	end
	ela = .2

	XCoordsPanel:SetScript("OnUpdate", cUpdate)
	SetMapToCurrentZone() -- gonna test this one if it helps on accurate coords

end

-- Datatext panels
function CreateLocDTPanels()

	-- Left coords Datatext panel
	--left_dtp:SetTemplate('Default', true)
	left_dtp:Width(DTPANELS_WIDTH)
	left_dtp:Height(LOCPANEL_HEIGHT)
	left_dtp:SetFrameStrata('LOW')
	left_dtp:SetParent(LocationPlusPanel) -- set to hide in combat with the Location Panel
	left_dtp:Show()

	-- Right coords Datatext panel
	--right_dtp:SetTemplate('Default', true)
	right_dtp:Width(DTPANELS_WIDTH)
	right_dtp:Height(LOCPANEL_HEIGHT)
	right_dtp:SetFrameStrata('LOW')
	right_dtp:SetParent(LocationPlusPanel) -- set to hide in combat with the Location Panel
	right_dtp:Show()
	
end

do
	left_dtp = CreateFrame('Frame', 'LeftCoordDtPanel', E.UIParent)
	DT:RegisterPanel(LeftCoordDtPanel, 1, 'ANCHOR_BOTTOM', 0, -4)
	right_dtp = CreateFrame('Frame', 'RightCoordDtPanel', E.UIParent)
	DT:RegisterPanel(RightCoordDtPanel, 1, 'ANCHOR_BOTTOM', 0, -4)
end

-- How to appear in datatext options
L['RightCoordDtPanel'] = L["LocationPlus Right Panel"];
L['LeftCoordDtPanel'] = L["LocationPlus Left Panel"];

-- Setting default datatexts
DF.datatexts.panels.RightCoordDtPanel = 'System'
DF.datatexts.panels.LeftCoordDtPanel = 'Durability'

function E:LocPPInit()
	CreateMainLocPanel()
	CreateLocCoordPanels()
		if loc_db.loc_datatxt then
			CreateLocDTPanels()
		end
	LocPlusUpdate()
end

ElvUI_Location:RegisterEvent("PLAYER_ENTERING_WORLD")
ElvUI_Location:SetScript("OnEvent",function(self, event)	-- test PLAYER_LOGIN
    if event == "PLAYER_ENTERING_WORLD" then
		if E.initialized then
			if loc_db.loc_enable then
				E:LocPPInit()
				E:LocCoordPanelFont()
				self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
				self:RegisterEvent("ZONE_CHANGED")
				self:RegisterEvent("ZONE_CHANGED_INDOORS")
				LocationPlusPanel:RegisterEvent("PLAYER_REGEN_DISABLED")
				LocationPlusPanel:RegisterEvent("PLAYER_REGEN_ENABLED")
				print(L["Location Plus "]..format("v|cff33ffff%s|r",ElvUI_Location.version)..L[" is loaded. Thank you for using it."])
			end
			ElvUI_Location:UnregisterEvent("PLAYER_ENTERING_WORLD")
		end
	end
end)

-- shows when the location panel is CtrlClicked and the DT panels are disabled
StaticPopupDialogs["LOC_DT_DISABLED"] = {
  text = L["LocationPlus Datatexts are disabled. Please go to the options to re-enable them."],
  button1 = "OK",
  timeout = 0,
  whileDead = true,
  hideOnEscape = true,
  showAlert = true,
  preferredIndex = 3, 
}